//Copyright (C) 2017 Potix Corporation. All Rights Reserved.
//History: Wed Dec 13 12:27:24 CST 2017
// Author: tomyeh
library link_matcher;

import "package:rikulo_commons/util.dart";

/// Parses a link for 2nd phase parsing of `LinkSyntax`
Match parseLink(String text, int start, int end) {
  //when called, it matches `](.*)`
  assert(text.substring(start, start + 2) == '](');
  assert(text[end - 1] == ')');
  int nParens = 0;
  String quote;
  int i = StringUtil.skipWhitespaces(text, start + 2),
      urlStart = i, urlEnd;
  for (;; ++i) {
    final cc = text[i];
    if (cc == ')') {
      if ((quote == null && --nParens < 0) || i + 1 >= end) {
        urlEnd = i;
        break;
      }
    } else if (quote == null) {
      if (cc == '(') {
        ++nParens;
      } else if (cc == '"' || cc == "'") {
        quote = cc;
      } else if (StringUtil.isChar(cc, whitespace: true)) {
        urlEnd = i;
        i = StringUtil.skipWhitespaces(text, i);
        break;
      }

    } else if (cc == quote) {
      quote = null;
    }
  }

  var cc = text[i];
  StringBuffer title;
  if (cc == '"') { //title
    title = new StringBuffer();
    for (;;) {
      if (++i >= end) return null; //wrong pattern

      cc = text[i];
      if (cc == '"') {
        i = StringUtil.skipWhitespaces(text, i + 1);
        if (text[i] != ')') return null; //wrong pattern
        break;
      }

      if (cc == r'\' && text[i + 1] == '"') {
        cc = '"';
        ++i; //skip `\"`
      }

      title.write(cc);
    }

  } else if (cc != ')') {
    return null;
  }
  return new _LinkMatch(text, text.substring(start, i + 1),
      text.substring(urlStart, urlEnd), title?.toString(), start, i + 1);
}

class _LinkMatch implements Match {
  final List<String> _groups;
  final String input;
  final int start, end;

  _LinkMatch(this.input, String matched, String url, String title,
    this.start, this.end): _groups = [matched, '', '', url, title];

  @override
  String operator[](int index) => _groups[index];
  @override
  String group(int index) => _groups[index];
  @override
  int get groupCount => 4;
  @override
  String get pattern => throw new UnsupportedError('');
  @override
  List<String> groups(List<int> indexes) => throw new UnsupportedError('');
}
